		RADIX dec
;
		list	p=16F630           ; list directive to define processor
        #include	<p16F630.inc>
        errorlevel -302			; suppress "register not in bank0, check page bits" message.	
;

#define guardtime 3
#define stimeout 500
#define ssleep 1
clockrate equ 4000000         ;define clock rate here
toconst	equ	(1+(stimeout*clockrate)/(4000*256*20))
slconst	equ	(1+(ssleep*clockrate)/(4*256*20))
guardconst	equ	(1+(guardtime*clockrate)/(4000*256*8))



vars	UDATA_SHR	0x20
delay	RES	2
shtstatus	RES 1
temp	RES 2
humi	RES 2
shttime	RES 3

#define SHT_START	0
#define SHT_MEAS_T	1
#define SHT_MEAS_RH	2
#define SHT_SLEEP	3

		EXTERN 	trw_sendpacket,trw_getpacket,trw_setrx,trw_setch,trw_init,trw_address
		EXTERN	s_connectionreset,s_measure_t,s_measure_rh,s_measure_wait
		EXTERN	s_measure_get
		EXTERN	message,value


reset	CODE	0
		goto start

ioffset	CODE 0x4

initint	CODE 0x5

start   clrf   	PORTA			; 
        clrf   	PORTC			; 
        movlw   B'00000111'		; Comparator off
        movwf   CMCON
		bsf STATUS,RP0 			; Bank 1
		call 3FFh 				; Get the cal value
		movwf OSCCAL 			; Calibrate
        movlw   B'00000000'		; Enable pull-up only in (none)
        movwf   WPUA
        movlw   B'00001111'		; Enable pull-ups, WDT 1:128 prescaler
        movwf   OPTION_REG
        movlw   B'00101001'     ; RA0,3,5 = input, RA1,2,4 = output
        movwf	TRISA
        movlw   B'00000001'     ; RC1,2,3,4,5 = output, RC0= input
        movwf	TRISC
		bcf STATUS,RP0 			; Bank 0

		call TXdelay			; Must wait at poweron before
		call trw_init			; init module
		call trw_address		; Set master address (to answer back)
		clrf shtstatus
		clrf shttime

main	call trw_setrx			; Enable rx
mloop	call trw_getpacket		; Get msg
		iorlw 0					; any ?
		btfsc STATUS,Z
		goto nomsg				; no, skip
		bsf PORTC,2
		call TXdelay 			; give the master some time to enable Rx (Rabbit CAN-045 waits 3ms after Tx)
		; insert real processing here, remove prior call when not needed
		movf shtstatus,W		; Return sensor status
		movwf message
		movf temp,W				; last temperature measurement
		movwf message+1
		movf temp+1,W
		movwf message+2
		movf humi,W				; last humidity measurement
		movwf message+3
		movf humi+1,W
		movwf message+4
		call trw_sendpacket		; and answer back
		call TXdelay 			; must wait >3ms so Tx goes out
		bcf PORTC,2
		goto main

; ensure it does not cross a 256 word boundary
nomsg	movlw HIGH nomsg
		movwf PCLATH
		movf shtstatus,W
		addwf PCL,f
		goto shtrst
		goto shtmeat
		goto shtmeah
		goto shtslp

shtrst	clrf shtstatus
		call s_connectionreset	; reset sensor
shtstrt	call s_measure_t		; measure temperature
		btfss STATUS,C			; reset sensor on error
		incf shtstatus,f		; wait for result
		movlw toconst			; init delay constant
		movwf shttime+1
		goto mloop
shtmeat	call chktmout
		btfsc STATUS,C			; reset on timeout
		goto shtrst
		call s_measure_wait
		btfsc STATUS,C			; return if need to wait
		goto mloop
		call s_measure_get		; get results
		movf value,W
		movwf temp
		movf value+1,W
		movwf temp+1
		call s_measure_rh		; measure humidity
		btfsc STATUS,C			; reset on error
		goto shtrst
		incf shtstatus,f		; next
		movlw toconst			; init delay constant
		movwf shttime+1
		goto mloop
shtmeah call chktmout
		btfsc STATUS,C			; reset on timeout
		goto shtrst
		call s_measure_wait
		btfsc STATUS,C			; return if need to wait
		goto mloop
		call s_measure_get		; get results
		movf value,W
		movwf humi
		movf value+1,W
		movwf humi+1
		incf shtstatus,f		; next (sleep for some time)
		movlw slconst			; init delay constant
		movwf shttime+1
		goto mloop
shtslp	call chktmout
		btfss STATUS,C			; return if need to wait
		goto mloop
		clrf shtstatus
		goto shtstrt			; restart measurement

chktmout:
		bcf STATUS,C
		decfsz shttime,f
		return
		decfsz shttime+1,f
		return
		bsf STATUS,C
		return

TXdelay	clrf delay
		movlw guardconst
		movwf delay+1
l1		goto l1a
l1a		goto l1b
l1b		nop
		decfsz delay,f
		goto l1
		decfsz delay+1,f
		goto l1
		return

		end
